<?php
/**
 * A general crtieria class
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2011 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class BaseCriteria extends BaseFrameworkSystem {
	/**
	 * Wrapper class name stored in config entry
	 */
	private $wrapperConfigEntry = '';

	/**
	 * Criteria to handle
	 */
	private $criteria = array();
	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);
	}

	/**
	 * Setter for wrapper class name
	 *
	 * @param	$wrapperConfigEntry		Configuration entry which hold the wrapper class' name
	 * @return	void
	 */
	public final function setWrapperConfigEntry ($wrapperConfigEntry) {
		$this->wrapperConfigEntry = (string) $wrapperConfigEntry;
	}

	/**
	 * Getter for wrapper class name
	 *
	 * @return	$wrapperConfigEntry		Configuration entry which hold the wrapper class' name
	 */
	public final function getWrapperConfigEntry () {
		return $this->wrapperConfigEntry;
	}

	/**
	 * Getter for criteria array
	 *
	 * @return	$criteria
	 */
	public final function getCriteriaArray () {
		return $this->criteria;
	}

	/**
	 * Add criteria
	 *
	 * @param	$criteriaKey	Criteria key
	 * @param	$criteriaValue	Criteria value
	 * @return	void
	 */
	public final function addCriteria ($criteriaKey, $criteriaValue) {
		$this->criteria[(string)$criteriaKey] = (string)$criteriaValue;
	}

	/**
	 * Add configured criteria
	 *
	 * @param	$criteriaKey	Criteria key
	 * @param	$configEntry	Configuration entry
	 * @return	void
	 */
	public final function addConfiguredCriteria ($criteriaKey, $configEntry) {
		// Add the configuration entry as a criteria
		$value = $this->getConfigInstance()->getConfigEntry($configEntry);
		$this->addCriteria($criteriaKey, $value);
	}

	/**
	 * Get criteria element or null if not found
	 *
	 * @param	$criteria	The criteria we want to have
	 * @return	$value		Whether the value of the critera or null
	 */
	public function getCriteriaElemnent ($criteria) {
		// Default is not found
		$value = NULL;

		// Is the criteria there?
		if (isset($this->criteria[$criteria])) {
			// Then use it
			$value = $this->criteria[$criteria];
		}

		// Return the value
		return $value;
	}

	/**
	 * Checks whether given array entry matches
	 *
	 * @param	$entryArray		Array with the entries to find
	 * @return	$matches		Whether the entry matches or not
	 */
	public function ifEntryMatches (array $entryArray) {
		// First nothing matches and nothing is counted
		$matches = false;
		$counted = 0;

		// Walk through all entries
		foreach ($entryArray as $key => $entry) {
			// Then walk through all search criteria
			foreach ($this->criteria as $criteriaKey => $criteriaValue) {
				// Is the element found and does it match?
				if (($key == $criteriaKey) && ($criteriaValue == $entry)) {
					// Then count this one up
					$counted++;
				} // END - if
			} // END - foreach
		} // END - foreach

		// Now check if expected criteria counts match
		$matches = ($counted == count($this->criteria));

		// Return the result
		return $matches;
	}

	/**
	 * "Getter" for a cache key
	 *
	 * @param	$onlyKeys	Only use these keys for a cache key
	 * @return	$cacheKey	The key suitable for the cache system
	 */
	public function getCacheKey ($onlyKeys = array()) {
		// Initialize the key
		$cacheKey = '';

		// Now walk through all criterias
		foreach ($this->criteria as $criteriaKey => $criteriaValue) {
			// Is the value in array or is $onlyKeys empty?
			if ((isset($onlyKeys[$criteriaKey])) || (count($onlyKeys) == 0)) {
				// Add the value URL encoded to avoid any trouble with special characters
				$cacheKey .= sprintf("%s=%s;",
					$criteriaKey,
					urlencode($criteriaValue)
				);
			} // END - if
		} // END - foreach

		// Remove last semicolon
		$cacheKey = substr($cacheKey, 0, -1);

		// Is the instance SearchCriteria?
		if ($this instanceof SearchCriteria) {
			// Check if 'limit' and 'skip' are in
			if (((isset($onlyKeys['limit'])) && (isset($onlyKeys['skip']))) || (count($onlyKeys) == 0)) {
				// Add limit and skip values
				$cacheKey .= sprintf(";%%limit%%=%s;%%skip%%=%s",
					$this->getLimit(),
					$this->getSkip()
				);
			} // END - if
		} // END - if

		// Return the cache key
		return $cacheKey;
	}
}

// [EOF]
?>
