<?php
/**
 * A general Registry
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2011 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class BaseRegistry extends BaseFrameworkSystem implements Registerable {
	/**
	 * Instance of this class
	 */
	private static $registryInstance = NULL;

	/**
	 * Instance registry
	 */
	private $instanceRegistry = array();

	/**
	 * Raw data entries (non-objects)
	 */
	private $rawEntries = array();

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);
	}

	/**
	 * Checks whether an instance key was found
	 *
	 * @param	$instanceKey	The key holding an instance in registry
	 * @return	$exists			Whether the key exists in registry
	 */
	public function instanceExists ($instanceKey) {
		// Does this key exists?
		$exists = (isset($this->instanceRegistry[$instanceKey]));

		// Return the result
		return $exists;
	}

	/**
	 * Adds/overwrites a new instance to the registry at the given key
	 *
	 * @param	$instanceKey		The key to identify the instance
	 * @param	$objectInstance		An instance we shall store
	 * @return	void
	 */
	public function addInstance ($instanceKey, Registerable $objectInstance) {
		$this->instanceRegistry[$instanceKey] = $objectInstance;
	}

	/**
	 * Getter for whole instance registry
	 *
	 * @return	$instanceRegistry	The whole instance registry array
	 */
	public final function getInstanceRegistry () {
		return $this->instanceRegistry;
	}

	/**
	 * Adds a new entry to the given list name. If you want to add objects
	 * please use addInstance() and getInstance() instead.
	 *
	 * @param	$key	The key to identify the whole list
	 * @param	$value	The value to be stored
	 * @return	void
	 */
	public final function addEntry ($key, $value) {
		// Simply add it
		$this->rawEntries[$key][] = $value;
	}

	/**
	 * Getter for entries or "sub entries"
	 *
	 * @return	$entries	An array with entries from this registry
	 */
	public final function getEntries ($key = NULL) {
		// Default is whole array
		$entries = $this->rawEntries;

		// Is $key set?
		if (!is_null($key)) {
			// Then use this entry
			$entries = $this->rawEntries[$key];
		} // END - if

		// Return the array
		return $entries;
	}

	/**
	 * "Getter" for an array of all entries for given key
	 *
	 * @param	$lookFor	The key to look for
	 * @return	$entry		An array with all keys
	 */
	public function getArrayFromKey ($lookFor) {
		// Init array
		$entry = array();

		// "Walk" over all entries
		foreach ($this->getEntries('object-name') as $key=>$value) {
			// Debug message
			//* DEBUG: */ $this->debugOutput('REGISTRY: Checking key=' . $key . ',value=' . $value . ',lookFor=' . $lookFor);

			// If $value matches the $lookFor, we need to look for more entries for this!
			if ($lookFor == $value) {
				// Look for more entries
				foreach ($this->getEntries() as $key2=>$value2) {
					// Debug message
					//* DEBUG: */ $this->debugOutput('REGISTRY: Checking key2=' . $key2 . ',value2=' . print_r($value2, true) . ',lookFor=' . $lookFor);

					// Both keys must match!
					if (($key == $key2) || (isset($value2[$key]))) {
						// Debug message
						//* DEBUG: */ $this->debugOutput('REGISTRY: Adding ' . $value2[$key] . ' ...');

						// Then add it
						$entry[$key2] = $value2[$key];
					} // END - if
				} // END - foreach

				// Skip further lookups
				break;
			} // END - if
		} // END - foreach

		// Return it
		return $entry;
	}

	/**
	 * Gets a registered instance or null if not found
	 *
	 * @param	$instanceKey		The key to identify the instance
	 * @return	$objectInstance		An instance we shall store
	 * @throws	NullPointerException	If the requested key is not found
	 */
	public function getInstance ($instanceKey) {
		// By default the instance is not in registry
		$objectInstance = NULL;

		// Is the instance there?
		if ($this->instanceExists($instanceKey)) {
			$objectInstance = $this->instanceRegistry[$instanceKey];
		} // END - if

		// Still not fetched?
		if (is_null($objectInstance)) {
			// This might happen if a non-registered key was requested
			throw new NullPointerException($this, self::EXCEPTION_IS_NULL_POINTER);
		} // END - if

		// Return the result
		return $objectInstance;
	}
}

// [EOF]
?>
